diff -u -r nginx-1.2.1/auto/options nginx-1.2.1.tproxy/auto/options
--- nginx-1.2.1/auto/options	2012-01-18 16:07:43.000000000 +0100
+++ nginx-1.2.1.tproxy/auto/options	2012-06-14 13:55:42.292821380 +0200
@@ -46,6 +46,7 @@
 
 NGX_FILE_AIO=NO
 NGX_IPV6=NO
+NGX_TPROXY=NO
 
 HTTP=YES
 
@@ -188,6 +189,7 @@
 
         --with-file-aio)                 NGX_FILE_AIO=YES           ;;
         --with-ipv6)                     NGX_IPV6=YES               ;;
+	--with-tproxy)                   NGX_TPROXY=YES             ;;
 
         --without-http)                  HTTP=NO                    ;;
         --without-http-cache)            HTTP_CACHE=NO              ;;
@@ -342,6 +344,7 @@
 
   --with-file-aio                    enable file AIO support
   --with-ipv6                        enable IPv6 support
+  --with-tproxy                      enable TPROXY support
 
   --with-http_ssl_module             enable ngx_http_ssl_module
   --with-http_realip_module          enable ngx_http_realip_module
diff -u -r nginx-1.2.1/auto/unix nginx-1.2.1.tproxy/auto/unix
--- nginx-1.2.1/auto/unix	2012-03-28 14:38:03.000000000 +0200
+++ nginx-1.2.1.tproxy/auto/unix	2012-06-14 14:01:57.929438002 +0200
@@ -496,6 +496,17 @@
     . auto/feature
 fi
 
+if [ $NGX_TPROXY = YES ]; then
+    ngx_feature="Linux TPROXY support"
+    ngx_feature_name="NGX_HAVE_TPROXY"
+    ngx_feature_run=no
+    ngx_feature_incs=
+    ngx_feature_path=
+    ngx_feature_libs=
+    ngx_feature_test=
+
+    . auto/feature
+fi
 
 ngx_feature="setproctitle()"
 ngx_feature_name="NGX_HAVE_SETPROCTITLE"
diff -u -r nginx-1.2.1/src/core/ngx_connection.c nginx-1.2.1.tproxy/src/core/ngx_connection.c
--- nginx-1.2.1/src/core/ngx_connection.c	2012-04-03 09:37:31.000000000 +0200
+++ nginx-1.2.1.tproxy/src/core/ngx_connection.c	2012-06-14 15:45:12.974841603 +0200
@@ -82,6 +82,10 @@
     ls->setfib = -1;
 #endif
 
+#if (NGX_HAVE_TPROXY)
+    ls->tproxy = 0;
+#endif
+
     return ls;
 }
 
@@ -259,6 +263,7 @@
 
         ls[i].deferred_accept = 1;
 #endif
+
     }
 
     return NGX_OK;
@@ -274,6 +279,9 @@
     ngx_log_t        *log;
     ngx_socket_t      s;
     ngx_listening_t  *ls;
+#if (NGX_HAVE_TPROXY)
+    int               tproxy;
+#endif
 
     reuseaddr = 1;
 #if (NGX_SUPPRESS_WARN)
@@ -368,6 +376,19 @@
                     return NGX_ERROR;
                 }
             }
+            
+#if (NGX_HAVE_TPROXY)
+            /* TPROXY support requires enabled (SOL_IP, IP_TRANSPARENT)       */
+            /* socket option to be able to send datagrams _from_ non-local IP */
+            tproxy = ls[i].tproxy;
+            if (setsockopt(s, SOL_IP, IP_TRANSPARENT, &tproxy,
+                     sizeof(tproxy)) == -1) {
+                ngx_log_error(NGX_LOG_EMERG, log, ngx_socket_errno,
+                     "setsockopt(SOL_IP, IP_TRANSPARENT) %V failed",
+                     &ls[i].addr_text);
+                return NGX_ERROR;
+            }
+#endif
 
             ngx_log_debug2(NGX_LOG_DEBUG_CORE, log, 0,
                            "bind() %V #%d ", &ls[i].addr_text, s);
diff -u -r nginx-1.2.1/src/core/ngx_connection.h nginx-1.2.1.tproxy/src/core/ngx_connection.h
--- nginx-1.2.1/src/core/ngx_connection.h	2012-01-18 16:07:43.000000000 +0100
+++ nginx-1.2.1.tproxy/src/core/ngx_connection.h	2012-06-14 14:04:09.546390482 +0200
@@ -72,6 +72,9 @@
     unsigned            deferred_accept:1;
     unsigned            delete_deferred:1;
     unsigned            add_deferred:1;
+#if (NGX_HAVE_TPROXY)
+    unsigned            tproxy:1;
+#endif
 #ifdef SO_ACCEPTFILTER
     char               *accept_filter;
 #endif
diff -u -r nginx-1.2.1/src/http/ngx_http.c nginx-1.2.1.tproxy/src/http/ngx_http.c
--- nginx-1.2.1/src/http/ngx_http.c	2012-04-03 09:37:31.000000000 +0200
+++ nginx-1.2.1.tproxy/src/http/ngx_http.c	2012-06-14 14:04:51.800791811 +0200
@@ -1786,6 +1786,10 @@
     ls->setfib = addr->opt.setfib;
 #endif
 
+#if (NGX_HAVE_TPROXY)
+    ls->tproxy = addr->opt.tproxy;
+#endif
+
     return ls;
 }
 
diff -u -r nginx-1.2.1/src/http/ngx_http_core_module.c nginx-1.2.1.tproxy/src/http/ngx_http_core_module.c
--- nginx-1.2.1/src/http/ngx_http_core_module.c	2012-06-04 13:58:12.000000000 +0200
+++ nginx-1.2.1.tproxy/src/http/ngx_http_core_module.c	2012-06-14 14:26:49.540964818 +0200
@@ -3865,6 +3865,9 @@
 #if (NGX_HAVE_SETFIB)
     lsopt.setfib = -1;
 #endif
+#if (NGX_HAVE_TPROXY)
+    lsopt.tproxy = 0;	
+#endif
     lsopt.wildcard = u.wildcard;
 
     (void) ngx_sock_ntop(&lsopt.u.sockaddr, lsopt.addr,
@@ -3885,6 +3888,17 @@
             continue;
         }
 
+        if (ngx_strcmp(value[n].data, "tproxy") == 0) {
+#if (NGX_HAVE_TPROXY)
+            lsopt.tproxy = 1;	
+#else
+            ngx_conf_log_error(NGX_LOG_WARN, cf, 0,
+                       "TPROXY support is not enabled, ignoring option \"tproxy\" in %V",
+                       &value[n]);
+#endif
+            continue;
+        }
+
 #if (NGX_HAVE_SETFIB)
         if (ngx_strncmp(value[n].data, "setfib=", 7) == 0) {
             lsopt.setfib = ngx_atoi(value[n].data + 7, value[n].len - 7);
diff -u -r nginx-1.2.1/src/http/ngx_http_core_module.h nginx-1.2.1.tproxy/src/http/ngx_http_core_module.h
--- nginx-1.2.1/src/http/ngx_http_core_module.h	2012-06-04 13:58:12.000000000 +0200
+++ nginx-1.2.1.tproxy/src/http/ngx_http_core_module.h	2012-06-14 14:06:39.548135512 +0200
@@ -79,6 +79,9 @@
     unsigned                   ipv6only:2;
 #endif
     unsigned                   so_keepalive:2;
+#if (NGX_HAVE_TPROXY)
+    unsigned                   tproxy:1;
+#endif
 
     int                        backlog;
     int                        rcvbuf;
